@@ -9,16 +9,22 @@
 #' @param report Should a table of results be printed to the console?
 #' @param drop Should clusters within which a model cannot be estimated be dropped?
 #' @param truncate Should outlying cluster-specific beta estimates be excluded?
+#' @param return.vcv Should a VCV matrix and the means of cluster-specific coefficient estimates be returned?
 #'
 #' @return A list with the elements
 #' \item{p.values}{A matrix of the estimated p-values.}
 #' \item{ci}{A matrix of confidence intervals.}
+#' \item{vcv.hat}{Optional: A cluster-level variance-covariance matrix for coefficient estimates.}
+#' \item{beta.bar}{Optional: A vector of means for cluster-specific coefficient estimates.}
 #' @author Justin Esarey
 #' @note Confidence intervals are centered on the cluster averaged estimate, which can diverge from original model estimates if clusters have different numbers of observations. Consequently, confidence intervals may not be centered on original model estimates. If drop = TRUE, any cluster for which all coefficients cannot be estimated will be automatically dropped from the analysis. If truncate = TRUE, any cluster for which any coefficient is more than 6 times the interquartile range from the cross-cluster mean will also be dropped as an outlier.
 #' @examples
 #' \dontrun{
 #' 
-#' # example: predict whether respondent has a university degree
+#' #####################################################################
+#' # example one: predict whether respondent has a university degree
+#' #####################################################################
+#' 
 #' require(effects)
 #' data(WVS)
 #' logit.model <- glm(degree ~ religion + gender + age, data=WVS, family=binomial(link="logit"))
@@ -27,14 +33,62 @@
 #' # compute cluster-adjusted p-values
 #' clust.im.p <- cluster.im.glm(logit.model, WVS, ~ country, report = T)
 #' 
+#' ############################################################################
+#' # example two: linear model of whether respondent has a university degree
+#' #              with interaction between gender and age + country FEs
+#' ############################################################################
+#' 
+#' WVS$degree.n <- as.numeric(WVS$degree)
+#' WVS$gender.n <- as.numeric(WVS$gender)
+#' WVS$genderXage <- WVS$gender.n * WVS$age
+#' lin.model <- glm(degree.n ~ gender.n + age + genderXage + religion + as.factor(country), data=WVS)
+#' 
+#' # compute marginal effect of male gender on probability of obtaining a university degree
+#' # using conventional standard errors
+#' age.vec <- seq(from=18, to=90, by=1)
+#' me.age <- coefficients(lin.model)[2] + coefficients(lin.model)[4]*age.vec
+#' plot(me.age ~ age.vec, type="l", ylim=c(-0.1, 0.1), xlab="age", 
+#'      ylab="ME of male gender on Pr(university degree)")
+#' se.age <- sqrt( vcov(lin.model)[2,2] + vcov(lin.model)[4,4]*(age.vec)^2 + 
+#'                 2*vcov(lin.model)[2,4]*age.vec)
+#' ci.h <- me.age + qt(0.975, lower.tail=T, df=lin.model$df.residual) * se.age
+#' ci.l <- me.age - qt(0.975, lower.tail=T, df=lin.model$df.residual) * se.age
+#' lines(ci.h ~ age.vec, lty=2)
+#' lines(ci.l ~ age.vec, lty=2)
+#' 
+#' 
+#' # cluster on country, compute CIs for marginal effect of gender on degree attainment
+#' # drop the FEs (absorbed into cluster-level coefficients)
+#' lin.model.n <- glm(degree.n ~ gender.n + age + genderXage + religion, data=WVS)
+#' clust.im.result <- cluster.im.glm(lin.model.n, WVS, ~ country, report = T, return.vcv = T)
+#' # compute ME using average of cluster-level estimates (CIs center on this)
+#' me.age.im <- clust.im.result$beta.bar[2] + clust.im.result$beta.bar[4]*age.vec
+#' se.age.im <- sqrt( clust.im.result$vcv[2,2] + clust.im.result$vcv[4,4]*(age.vec)^2 +
+#'                    2*clust.im.result$vcv[2,4]*age.vec)
+#' # center the CIs on the ME using average of cluster-level estimates
+#' # important: divide by sqrt(G) to convert SE of cluster-level estimates 
+#' #            into SE of the mean, where G = number of clusters
+#' G <- length(unique(WVS$country))
+#' ci.h.im <- me.age.im + qt(0.975, lower.tail=T, df=(G-1)) * se.age.im/sqrt(G)
+#' ci.l.im <- me.age.im - qt(0.975, lower.tail=T, df=(G-1)) * se.age.im/sqrt(G)
+#' plot(me.age.im ~ age.vec, type="l", ylim=c(-0.2, 0.2), xlab="age", 
+#'      ylab="ME of male gender on Pr(university degree)")
+#' lines(ci.h.im ~ age.vec, lty=2)
+#' lines(ci.l.im ~ age.vec, lty=2)
+#' # for comparison, here's the ME estimate and CIs from the baseline model
+#' lines(me.age ~ age.vec, lty=1, col="gray")
+#' lines(ci.h ~ age.vec, lty=3, col="gray")
+#' lines(ci.l ~ age.vec, lty=3, col="gray")
+#' 
 #' }
 #' @rdname cluster.im.glm
+#' @references Esarey, Justin, and Andrew Menger. 2017. "Practical and Effective Approaches to Dealing with Clustered Data." \emph{Political Science Research and Methods} forthcoming: 1-35. <URL:http://jee3.web.rice.edu/cluster-paper.pdf>.
 #' @references Ibragimov, Rustam, and Ulrich K. Muller. 2010. "t-Statistic Based Correlation and Heterogeneity Robust Inference." \emph{Journal of Business & Economic Statistics} 28(4): 453-468. <DOI:10.1198/jbes.2009.08046>.
 #' @import stats
 #' @importFrom utils write.table
 #' @export
 
-cluster.im.glm<-function(mod, dat, cluster, ci.level = 0.95, report = TRUE, drop = FALSE, truncate = FALSE){
+cluster.im.glm<-function(mod, dat, cluster, ci.level = 0.95, report = TRUE, drop = FALSE, truncate = FALSE, return.vcv = FALSE){
   
   
   form <- mod$formula                                                   # what is the formula of this model?
@@ -124,6 +178,8 @@ cluster.im.glm<-function(mod, dat, cluster, ci.level = 0.95, report = TRUE, drop
 
   t.hat <- sqrt(G) * (b.hat / s.hat)                        # calculate t-statistic
   
+  names(b.hat) <- ind.variables
+  
   # compute p-val based on # of clusters
   p.out <- 2*pmin( pt(t.hat, df = G-1, lower.tail = TRUE), pt(t.hat, df = G-1, lower.tail = FALSE) )
   
@@ -165,8 +221,10 @@ cluster.im.glm<-function(mod, dat, cluster, ci.level = 0.95, report = TRUE, drop
  
   
   out.list<-list()
-  out.list[["p.values"]]<-out
-  out.list[["ci"]]<-out.ci
+  out.list[["p.values"]] <- out
+  out.list[["ci"]] <- out.ci
+  if(return.vcv == TRUE){out.list[["vcv.hat"]] <- vcv.hat}
+  if(return.vcv == TRUE){out.list[["beta.bar"]] <- b.hat}
   return(invisible(out.list))
   
 }
